#include <AIV/HedgeTrades.mqh>

input double InpSreadFactor = 1;
input double InpVolume = 0.01;
input string InpAC = "EURUSD.m";
input string InpBC = "GBPUSD.m";
input string InpAB = "EURGBP.m";

double iCostAC;
double iCostBC;
double iCostAB;

SSymbol iSymAC = {0};
SSymbol iSymBC = {0};
SSymbol iSymAB = {0};

int OnInit()
{
    long chart = ChartID();

    iCustom(InpBC, PERIOD_M1, "AIV/Spy", chart, 1);
    iCustom(InpAC, PERIOD_M1, "AIV/Spy", chart, 2);
    iCustom(InpAB, PERIOD_M1, "AIV/Spy", chart, 3);

    iCostAC = SymbolInfoDouble(InpAC, SYMBOL_TRADE_TICK_VALUE);
    iCostBC = SymbolInfoDouble(InpBC, SYMBOL_TRADE_TICK_VALUE);
    iCostAB = SymbolInfoDouble(InpAB, SYMBOL_TRADE_TICK_VALUE);

    return 0;
}

void Trade()
{
    int positions = PositionsTotal();

    double incomeAC = getPositionsIncome(InpAC);
    double incomeBC = getPositionsIncome(InpBC);
    double incomeAB = getPositionsIncome(InpAB);

    if (positions > 0)
    {
        bool A = incomeAC + incomeBC + incomeAB > 0;

        if (A) 
        {
            closePositions();
        }
    }

    if (positions == 0)
    {
        int state = isGap();
        double sizeAC = InpVolume; // * iCostBC * iCostAB;
        double sizeBC = InpVolume; // * iCostAC * iCostAB;
        double sizeAB = InpVolume; // * iCostAC * iCostBC;

        if (state == 1) 
        {
            openSimplePosition(InpAC, -1, sizeAC);
            openSimplePosition(InpBC, -1, sizeBC);
            openSimplePosition(InpAB, 1, sizeAB);
        }

        if (state == -1) 
        {
            openSimplePosition(InpAC, 1, sizeAC);
            openSimplePosition(InpBC, 1, sizeBC);
            openSimplePosition(InpAB, -1, sizeAB);
        }
    }
}

int isGap()
{
    bool EU = iSymAC.mUnit > 0;
    bool GU = iSymBC.mUnit > 0;
    bool EG = iSymAB.mUnit > 0;

    if (!EU || !GU || !EG)
    {
        return 0;
    }

    double X = iSymAC.mUnit / iSymBC.mUnit;
    double Y = iSymAB.mUnit;
    double spread = InpSreadFactor * (iSymAC.mMean + iSymBC.mMean + iSymAB.mMean);

    if (X - Y - spread > 0)
    {
        return 1;
    }

    if (Y - X - spread > 0)
    {
        return -1;
    }

    return 0;
}

void OnChartEvent(const int id, const long& lparam, const double& dparam, const string& sparam)
{
    string message[];
    
    StringSplit(sparam, ':', message);

    int event = id - CHARTEVENT_CUSTOM;
    string symbol = message[0];
    double close = StringToDouble(message[1]);
    double spread = StringToDouble(message[2]) * SymbolInfoDouble(symbol, SYMBOL_POINT);

    if (id > CHARTEVENT_CUSTOM) 
    {
        if (symbol == InpAC)
        {
            iSymAC.mName = symbol;
            iSymAC.mUnit = close;
            iSymAC.mMean = spread;
        }

        if (symbol == InpBC)
        {
            iSymBC.mName = symbol;
            iSymBC.mUnit = close;
            iSymBC.mMean = spread;
        }

        if (symbol == InpAB)
        {
            iSymAB.mName = symbol;
            iSymAB.mUnit = close;
            iSymAB.mMean = spread;
        }

        bool AC = iSymAC.mUnit > 0;
        bool BC = iSymBC.mUnit > 0;
        bool AB = iSymAB.mUnit > 0;

        if (AC && BC && AB)
        {
            Trade();
        }
    }
}